@@ -1,13 +1,18 @@
 #include <config.h>
 
+/* #define DEBUG Don't even think defining it:) */
+
 #ifdef STDC_HEADERS
 # include <string.h>
 # include <stdlib.h>
 # include <stdio.h>
 #endif
 
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
 #include <time.h>
-#include "getopt.h"
 
 #ifndef FUNCTIONS_H
 # define FUNCTIONS_H
@@ -22,7 +27,9 @@
 # define ZIP
 #endif
 
-
+#ifdef HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
 
 #ifdef HAVE_SIGNAL_H
 # include <signal.h>
@@ -49,10 +56,6 @@
 # include <sys/types.h>
 #endif
 
-#ifdef HAVE_SYS_WAIT_H
-# include <sys/wait.h>
-#endif
-
 #ifdef HAVE_PWD_H
 # include <pwd.h>
 #endif
@@ -71,8 +74,12 @@
 
 
 /* defines */
-#define BITS4 2
-#define BITS7 1
+#define SALT_SIZE 20
+#define RESERVED 20
+
+#define MD5HASH 3
+#define SHA1HASH 2
+#define BITS4 1
 #define BITS8 0
 
 #define TRUE 1
@@ -151,3 +158,6 @@ extern char *crypt(); /* libufc */
 # define TERMIO struct sgttyb
 # define USE_SGTTY
 #endif
+
+#define CRYPT 50
+#define DES_COMPAT 51
