@@ -1,16 +1,17 @@
-#define MAX_KEY_LEN 300
+#define MAX_KEY_LEN 513
 
-char * get_password(int numofchar,int bitmode,int mode,int type, unsigned int *len);
+char * get_password(int numofchar,int bitmode,int mode,int type, unsigned int *len, void * salt);
 int show_mode(int ende, int stype,int smode,char *output,int bitmode);
 char** read_key_file(char * file, int* num);
 char *get_cfile(int uid, char*);
 
-char* get_modes_name(int mode);
+int is_block_algorithm_local(int algorithm);
+char* get_algorithms_name_local(int);
 
 char *my_getpass();
 
 int check_file_head(char * filename, unsigned int *algorithm, unsigned int *mode, unsigned int *bit7_flag,unsigned int stream_flag);
-int write_file_head(int filedes, unsigned int algorithm, unsigned int mode, unsigned int bit7_flag);
+int write_file_head(FILE* filedes, unsigned int algorithm, unsigned int mode, unsigned int bit7_flag, void* salt);
 
 void cleanUp();
 void shandler(int signal);
