@@ -12,61 +12,51 @@
  *    You should have received a copy of the GNU General Public License
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-*/
+ */
 
 /* Functions for that may not exist in some unices */
 
-#include "defines.h"
+/* $Id: functions.c,v 1.1.1.1 1999/10/15 22:49:23 nmav Exp $ */
+
+#ifndef DEFINES_H
+#define DEFINES_H
+#include <defines.h>
+#endif
+
 #ifndef FUNCTIONS_H
-# define FUNCTIONS_H
-# include "functions.h"
+#define FUNCTIONS_H
+#include "functions.h"
 #endif
 
-void Bzero( void *s, int n) {
-#ifdef HAVE_MEMSET
-  memset( s, '\0', n);
-#else
-# ifdef HAVE_BZERO
-  bzero( s, n);
-# else
-char *stmp=s;
-
- for (int i=0;i<n;i++)
-  stmp[i]='\0';
-  
-# endif  
-#endif
-}
-
-Sigfunc *Signal( int signo, Sigfunc *func)
+Sigfunc *
+ Signal(int signo, Sigfunc * func)
 {
-# ifdef HAVE_SIGACTION
+#ifdef HAVE_SIGACTION
 
-   struct sigaction act, oact;
-   
-   act.sa_handler = func;
-   sigemptyset( &act.sa_mask);
-   act.sa_flags=0;
-   if (signo==SIGALRM) {
+	struct sigaction act, oact;
+
+	act.sa_handler = func;
+	sigemptyset(&act.sa_mask);
+	act.sa_flags = 0;
+	if (signo == SIGALRM) {
 #ifdef SA_INTERRUPT
-     act.sa_flags |= SA_INTERRUPT; /* SunOs 4.x */
+		act.sa_flags |= SA_INTERRUPT;	/* SunOs 4.x */
 #endif
-   }
-   else {
+	} else {
 #ifdef SA_RESTART
-     act.sa_flags |= SA_RESTART; /* SVR4, 4.4BSD */
+		act.sa_flags |= SA_RESTART;	/* SVR4, 4.4BSD */
 #endif
-   }
-   
-   if (sigaction(signo, &act, &oact) < 0)
-       return (SIG_ERR);
-   return (oact.sa_handler);
+	}
 
-# else
-#  ifdef HAVE_SIGNAL
-   return signal(signo, func);
-#  else
-   return (Sigfunc*)0; /* Do nothing */
-#  endif
-# endif
+	if (sigaction(signo, &act, &oact) < 0)
+		return (SIG_ERR);
+	return (oact.sa_handler);
+
+#else
+#ifdef HAVE_SIGNAL
+	return signal(signo, func);
+#else
+	return (Sigfunc *) 0;	/* Do nothing */
+#endif
+#endif
 }
