@@ -8,38 +8,44 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 67 "mcrypt.gaa"
+#line 77 "mcrypt.gaa"
 	char **input;
-#line 66 "mcrypt.gaa"
+#line 76 "mcrypt.gaa"
 	int size;
-#line 58 "mcrypt.gaa"
+#line 67 "mcrypt.gaa"
 	char *mode;
-#line 55 "mcrypt.gaa"
+#line 64 "mcrypt.gaa"
 	char **keys;
-#line 54 "mcrypt.gaa"
+#line 63 "mcrypt.gaa"
 	int keysize;
-#line 48 "mcrypt.gaa"
+#line 57 "mcrypt.gaa"
 	char *algorithm;
-#line 45 "mcrypt.gaa"
+#line 54 "mcrypt.gaa"
 	char *config_file;
-#line 44 "mcrypt.gaa"
+#line 53 "mcrypt.gaa"
 	int config;
-#line 41 "mcrypt.gaa"
+#line 50 "mcrypt.gaa"
 	char *keyfile;
-#line 37 "mcrypt.gaa"
-	int sunlink_flag;
-#line 33 "mcrypt.gaa"
+#line 46 "mcrypt.gaa"
 	int unlink_flag;
-#line 30 "mcrypt.gaa"
-	int bit;
-#line 27 "mcrypt.gaa"
+#line 43 "mcrypt.gaa"
+	char *kmode;
+#line 40 "mcrypt.gaa"
 	int hexflag;
-#line 24 "mcrypt.gaa"
+#line 37 "mcrypt.gaa"
 	int ed_specified;
-#line 23 "mcrypt.gaa"
+#line 36 "mcrypt.gaa"
 	int ein;
-#line 22 "mcrypt.gaa"
+#line 35 "mcrypt.gaa"
 	int din;
+#line 32 "mcrypt.gaa"
+	int double_check;
+#line 29 "mcrypt.gaa"
+	int flush;
+#line 25 "mcrypt.gaa"
+	int noecho;
+#line 22 "mcrypt.gaa"
+	int nolock;
 #line 19 "mcrypt.gaa"
 	int bzipflag;
 #line 16 "mcrypt.gaa"
