@@ -1,4 +1,11 @@
 #include <stdio.h>
+#include <string.h>
+
+#ifndef GAA_NO_WIN32
+#if defined(_WIN32) || defined(__WIN32__) || defined(WIN32) || defined(WINDOWS)
+#define GAA_WIN32
+#endif
+#endif
 
 void __gaa_helpsingle(char short_name, char *name, 
 	char *arg_desc, char *opt_help)
@@ -103,16 +110,20 @@ void gaa_help()
 	__gaa_helpsingle('b', "bare", "", "Do not keep algorithm information in the encrypted""file.");
 	__gaa_helpsingle('z', "gzip", "", "Compress before encryption.");
 	__gaa_helpsingle('p', "bzip2", "", "Compress before encryption.");
+	__gaa_helpsingle('n', "nolock", "", "Do not lock files.");
+	__gaa_helpsingle(0, "echo", "", "Echo asterisks.");
+	__gaa_helpsingle(0, "flush", "", "Immediately flush the output");
+	__gaa_helpsingle('l', "doublecheck", "", "Double check passwords.");
 	__gaa_helpsingle('d', "decrypt", "", "decrypts.");
 	__gaa_helpsingle('x', "hex", "", "Keyword is in hex.");
-	__gaa_helpsingle('i', "bit", "", "keyword bit mode (4,7 or 8).");
+	__gaa_helpsingle('o', "keymode", """kmode"" ", "keyword mode (4, 8 or hash).");
 	__gaa_helpsingle('u', "unlink", "", "unlink the input file after""encryption/decryption.");
-	__gaa_helpsingle('s', "sunlink", "", "unlink (and destroy) the input file after""encryption/decryption.");
 	__gaa_helpsingle('f', "keyfile", """keyfile"" ", "file to read the keyword from.");
 	__gaa_helpsingle('c', "config", """config"" ", "Use configuration file FILE.");
 	__gaa_helpsingle('a', "algorithm", """algorithm"" ", "specify the encryption and decryption algorithm.""Legal parameters are: 3way, des, 3des, gost, tea,""safer64, safer128, cast128, crypt, twofish""and blowfish. (Default is blowfish)");
 	__gaa_helpsingle('k', "key", """key1 key2...keyN"" ", "specify the key(s)");
 	__gaa_helpsingle('m', "mode", """mode"" ", "specify the encryption and decryption mode.""CBC, ECB or CFB. CBC is the default.");
+	__gaa_helpsingle(0, "list", "", "Print a list of the supported algorithms and modes.");
 	__gaa_helpsingle('h', "help", "", "prints this help");
 	__gaa_helpsingle('v', "version", "", "prints the version number");
 	__gaa_helpsingle('L', "license", "", "displays license information.");
@@ -130,38 +141,44 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 67 "mcrypt.gaa"
+#line 77 "mcrypt.gaa"
 	char **input;
-#line 66 "mcrypt.gaa"
+#line 76 "mcrypt.gaa"
 	int size;
-#line 58 "mcrypt.gaa"
+#line 67 "mcrypt.gaa"
 	char *mode;
-#line 55 "mcrypt.gaa"
+#line 64 "mcrypt.gaa"
 	char **keys;
-#line 54 "mcrypt.gaa"
+#line 63 "mcrypt.gaa"
 	int keysize;
-#line 48 "mcrypt.gaa"
+#line 57 "mcrypt.gaa"
 	char *algorithm;
-#line 45 "mcrypt.gaa"
+#line 54 "mcrypt.gaa"
 	char *config_file;
-#line 44 "mcrypt.gaa"
+#line 53 "mcrypt.gaa"
 	int config;
-#line 41 "mcrypt.gaa"
+#line 50 "mcrypt.gaa"
 	char *keyfile;
-#line 37 "mcrypt.gaa"
-	int sunlink_flag;
-#line 33 "mcrypt.gaa"
+#line 46 "mcrypt.gaa"
 	int unlink_flag;
-#line 30 "mcrypt.gaa"
-	int bit;
-#line 27 "mcrypt.gaa"
+#line 43 "mcrypt.gaa"
+	char *kmode;
+#line 40 "mcrypt.gaa"
 	int hexflag;
-#line 24 "mcrypt.gaa"
+#line 37 "mcrypt.gaa"
 	int ed_specified;
-#line 23 "mcrypt.gaa"
+#line 36 "mcrypt.gaa"
 	int ein;
-#line 22 "mcrypt.gaa"
+#line 35 "mcrypt.gaa"
 	int din;
+#line 32 "mcrypt.gaa"
+	int double_check;
+#line 29 "mcrypt.gaa"
+	int flush;
+#line 25 "mcrypt.gaa"
+	int noecho;
+#line 22 "mcrypt.gaa"
+	int nolock;
 #line 19 "mcrypt.gaa"
 	int bzipflag;
 #line 16 "mcrypt.gaa"
@@ -228,27 +245,31 @@ int gaa_error = 0;
 #define GAA_MULTIPLE_OPTION     3
 
 #define GAA_REST                0
-#define GAA_NB_OPTION           20
+#define GAA_NB_OPTION           24
 #define GAAOPTID_license	1
 #define GAAOPTID_version	2
 #define GAAOPTID_help	3
-#define GAAOPTID_mode	4
-#define GAAOPTID_key	5
-#define GAAOPTID_algorithm	6
-#define GAAOPTID_config	7
-#define GAAOPTID_keyfile	8
-#define GAAOPTID_sunlink	9
+#define GAAOPTID_list	4
+#define GAAOPTID_mode	5
+#define GAAOPTID_key	6
+#define GAAOPTID_algorithm	7
+#define GAAOPTID_config	8
+#define GAAOPTID_keyfile	9
 #define GAAOPTID_unlink	10
-#define GAAOPTID_bit	11
+#define GAAOPTID_keymode	11
 #define GAAOPTID_hex	12
 #define GAAOPTID_decrypt	13
-#define GAAOPTID_bzip2	14
-#define GAAOPTID_gzip	15
-#define GAAOPTID_bare	16
-#define GAAOPTID_random	17
-#define GAAOPTID_force	18
-#define GAAOPTID_quiet	19
-#define GAAOPTID_verbose	20
+#define GAAOPTID_doublecheck	14
+#define GAAOPTID_flush	15
+#define GAAOPTID_echo	16
+#define GAAOPTID_nolock	17
+#define GAAOPTID_bzip2	18
+#define GAAOPTID_gzip	19
+#define GAAOPTID_bare	20
+#define GAAOPTID_random	21
+#define GAAOPTID_force	22
+#define GAAOPTID_quiet	23
+#define GAAOPTID_verbose	24
 
 #line 168 "gaa.skel"
 
@@ -431,18 +452,6 @@ float gaa_getfloat(char *arg)
 }
 /* option structures */
 
-struct GAAOPTION_license 
-{
-};
-
-struct GAAOPTION_version 
-{
-};
-
-struct GAAOPTION_help 
-{
-};
-
 struct GAAOPTION_mode 
 {
 	char* arg1;
@@ -473,55 +482,11 @@ struct GAAOPTION_keyfile
 	int size1;
 };
 
-struct GAAOPTION_sunlink 
+struct GAAOPTION_keymode 
 {
-};
-
-struct GAAOPTION_unlink 
-{
-};
-
-struct GAAOPTION_bit 
-{
-	int arg1;
+	char* arg1;
 	int size1;
 };
-
-struct GAAOPTION_hex 
-{
-};
-
-struct GAAOPTION_decrypt 
-{
-};
-
-struct GAAOPTION_bzip2 
-{
-};
-
-struct GAAOPTION_gzip 
-{
-};
-
-struct GAAOPTION_bare 
-{
-};
-
-struct GAAOPTION_random 
-{
-};
-
-struct GAAOPTION_force 
-{
-};
-
-struct GAAOPTION_quiet 
-{
-};
-
-struct GAAOPTION_verbose 
-{
-};
 #define GAA_REST_EXISTS
 
 struct GAAREST
@@ -533,6 +498,10 @@ struct GAAREST
 #line 349 "gaa.skel"
 int gaa_is_an_argument(char *str)
 {
+#ifdef GAA_WIN32
+    if(str[0] == '/' && str[1] != 0)
+	return GAA_MULTIPLE_OPTION;
+#endif
     if(str[0] != '-')
         return GAA_NOT_AN_OPTION;
     if(str[1] == 0)
@@ -560,16 +529,20 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECK1STR("a", GAAOPTID_algorithm);
 			GAA_CHECK1STR("c", GAAOPTID_config);
 			GAA_CHECK1STR("f", GAAOPTID_keyfile);
-			GAA_CHECK1STR("i", GAAOPTID_bit);
+			GAA_CHECK1STR("o", GAAOPTID_keymode);
         case GAA_MULTIPLE_OPTION:
 #line 375 "gaa.skel"
 			GAA_CHECK1STR("L", GAAOPTID_license);
 			GAA_CHECK1STR("v", GAAOPTID_version);
 			GAA_CHECK1STR("h", GAAOPTID_help);
-			GAA_CHECK1STR("s", GAAOPTID_sunlink);
+			GAA_CHECK1STR("", GAAOPTID_list);
 			GAA_CHECK1STR("u", GAAOPTID_unlink);
 			GAA_CHECK1STR("x", GAAOPTID_hex);
 			GAA_CHECK1STR("d", GAAOPTID_decrypt);
+			GAA_CHECK1STR("l", GAAOPTID_doublecheck);
+			GAA_CHECK1STR("", GAAOPTID_flush);
+			GAA_CHECK1STR("", GAAOPTID_echo);
+			GAA_CHECK1STR("n", GAAOPTID_nolock);
 			GAA_CHECK1STR("p", GAAOPTID_bzip2);
 			GAA_CHECK1STR("z", GAAOPTID_gzip);
 			GAA_CHECK1STR("b", GAAOPTID_bare);
@@ -584,16 +557,20 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("license", GAAOPTID_license);
 			GAA_CHECKSTR("version", GAAOPTID_version);
 			GAA_CHECKSTR("help", GAAOPTID_help);
+			GAA_CHECKSTR("list", GAAOPTID_list);
 			GAA_CHECKSTR("mode", GAAOPTID_mode);
 			GAA_CHECKSTR("key", GAAOPTID_key);
 			GAA_CHECKSTR("algorithm", GAAOPTID_algorithm);
 			GAA_CHECKSTR("config", GAAOPTID_config);
 			GAA_CHECKSTR("keyfile", GAAOPTID_keyfile);
-			GAA_CHECKSTR("sunlink", GAAOPTID_sunlink);
 			GAA_CHECKSTR("unlink", GAAOPTID_unlink);
-			GAA_CHECKSTR("bit", GAAOPTID_bit);
+			GAA_CHECKSTR("keymode", GAAOPTID_keymode);
 			GAA_CHECKSTR("hex", GAAOPTID_hex);
 			GAA_CHECKSTR("decrypt", GAAOPTID_decrypt);
+			GAA_CHECKSTR("doublecheck", GAAOPTID_doublecheck);
+			GAA_CHECKSTR("flush", GAAOPTID_flush);
+			GAA_CHECKSTR("echo", GAAOPTID_echo);
+			GAA_CHECKSTR("nolock", GAAOPTID_nolock);
 			GAA_CHECKSTR("bzip2", GAAOPTID_bzip2);
 			GAA_CHECKSTR("gzip", GAAOPTID_gzip);
 			GAA_CHECKSTR("bare", GAAOPTID_bare);
@@ -613,26 +590,12 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 {
     int OK = 0;
     int gaa_last_non_option;
-	struct GAAOPTION_license GAATMP_license;
-	struct GAAOPTION_version GAATMP_version;
-	struct GAAOPTION_help GAATMP_help;
 	struct GAAOPTION_mode GAATMP_mode;
 	struct GAAOPTION_key GAATMP_key;
 	struct GAAOPTION_algorithm GAATMP_algorithm;
 	struct GAAOPTION_config GAATMP_config;
 	struct GAAOPTION_keyfile GAATMP_keyfile;
-	struct GAAOPTION_sunlink GAATMP_sunlink;
-	struct GAAOPTION_unlink GAATMP_unlink;
-	struct GAAOPTION_bit GAATMP_bit;
-	struct GAAOPTION_hex GAATMP_hex;
-	struct GAAOPTION_decrypt GAATMP_decrypt;
-	struct GAAOPTION_bzip2 GAATMP_bzip2;
-	struct GAAOPTION_gzip GAATMP_gzip;
-	struct GAAOPTION_bare GAATMP_bare;
-	struct GAAOPTION_random GAATMP_random;
-	struct GAAOPTION_force GAATMP_force;
-	struct GAAOPTION_quiet GAATMP_quiet;
-	struct GAAOPTION_verbose GAATMP_verbose;
+	struct GAAOPTION_keymode GAATMP_keymode;
 
 #line 393 "gaa.skel"
 #ifdef GAA_REST_EXISTS
@@ -655,23 +618,30 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
     {
 	case GAAOPTID_license:
 	OK = 0;
-#line 64 "mcrypt.gaa"
+#line 74 "mcrypt.gaa"
 { mcrypt_license(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_version:
 	OK = 0;
-#line 63 "mcrypt.gaa"
+#line 73 "mcrypt.gaa"
 { mcrypt_version(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_help:
 	OK = 0;
-#line 62 "mcrypt.gaa"
+#line 72 "mcrypt.gaa"
 { usage(); exit(0); ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_list:
+	OK = 0;
+#line 71 "mcrypt.gaa"
+{ print_list(); exit(0); ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_mode:
@@ -679,7 +649,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_mode.arg1, gaa_getstr, GAATMP_mode.size1);
 		gaa_index++;
-#line 59 "mcrypt.gaa"
+#line 68 "mcrypt.gaa"
 { gaaval->mode = GAATMP_mode.arg1 ;};
 
 		return GAA_OK;
@@ -687,7 +657,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 	case GAAOPTID_key:
 	OK = 0;
 		GAA_LIST_FILL(GAATMP_key.arg1, gaa_getstr, char*, GAATMP_key.size1);
-#line 56 "mcrypt.gaa"
+#line 65 "mcrypt.gaa"
 { gaaval->keys = GAATMP_key.arg1; gaaval->keysize = GAATMP_key.size1 ;};
 
 		return GAA_OK;
@@ -697,7 +667,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithm.arg1, gaa_getstr, GAATMP_algorithm.size1);
 		gaa_index++;
-#line 49 "mcrypt.gaa"
+#line 58 "mcrypt.gaa"
 { gaaval->algorithm = GAATMP_algorithm.arg1 ;};
 
 		return GAA_OK;
@@ -707,7 +677,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_config.arg1, gaa_getstr, GAATMP_config.size1);
 		gaa_index++;
-#line 46 "mcrypt.gaa"
+#line 55 "mcrypt.gaa"
 { gaaval->config=1; if (GAATMP_config.size1==1) gaaval->config_file = GAATMP_config.arg1 ;};
 
 		return GAA_OK;
@@ -717,47 +687,68 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keyfile.arg1, gaa_getstr, GAATMP_keyfile.size1);
 		gaa_index++;
-#line 42 "mcrypt.gaa"
+#line 51 "mcrypt.gaa"
 { gaaval->keyfile = GAATMP_keyfile.arg1 ;};
 
-		return GAA_OK;
-		break;
-	case GAAOPTID_sunlink:
-	OK = 0;
-#line 38 "mcrypt.gaa"
-{ gaaval->sunlink_flag = 1 ;};
-
 		return GAA_OK;
 		break;
 	case GAAOPTID_unlink:
 	OK = 0;
-#line 34 "mcrypt.gaa"
+#line 47 "mcrypt.gaa"
 { gaaval->unlink_flag = 1 ;};
 
 		return GAA_OK;
 		break;
-	case GAAOPTID_bit:
+	case GAAOPTID_keymode:
 	OK = 0;
 		GAA_TESTMOREARGS;
-		GAA_FILL(GAATMP_bit.arg1, gaa_getint, GAATMP_bit.size1);
+		GAA_FILL(GAATMP_keymode.arg1, gaa_getstr, GAATMP_keymode.size1);
 		gaa_index++;
-#line 31 "mcrypt.gaa"
-{ gaaval->bit = GAATMP_bit.arg1 ;};
+#line 44 "mcrypt.gaa"
+{ gaaval->kmode = GAATMP_keymode.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_hex:
 	OK = 0;
-#line 28 "mcrypt.gaa"
+#line 41 "mcrypt.gaa"
 { gaaval->hexflag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_decrypt:
 	OK = 0;
-#line 25 "mcrypt.gaa"
+#line 38 "mcrypt.gaa"
 { gaaval->ed_specified=1; gaaval->din = 1; gaaval->ein=0; ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_doublecheck:
+	OK = 0;
+#line 33 "mcrypt.gaa"
+{ gaaval->double_check = 1 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_flush:
+	OK = 0;
+#line 30 "mcrypt.gaa"
+{ gaaval->flush = 1 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_echo:
+	OK = 0;
+#line 26 "mcrypt.gaa"
+{ gaaval->noecho = 0 ;};
+
+		return GAA_OK;
+		break;
+	case GAAOPTID_nolock:
+	OK = 0;
+#line 23 "mcrypt.gaa"
+{ gaaval->nolock = 1 ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_bzip2:
@@ -811,7 +802,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		break;
 	case GAA_REST:
 		GAA_OPTIONALLIST_FILL(GAAREST_tmp.arg1, gaa_getstr, char*, GAAREST_tmp.size1);
-#line 68 "mcrypt.gaa"
+#line 78 "mcrypt.gaa"
 { gaaval->input = GAAREST_tmp.arg1; gaaval->size = GAAREST_tmp.size1 ;};
 
 		return GAA_OK;
@@ -839,12 +830,13 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     if(inited == 0)
     {
 
-#line 70 "mcrypt.gaa"
+#line 80 "mcrypt.gaa"
 { gaaval->force=0; gaaval->quiet=1; gaaval->real_random_flag=0; gaaval->config=0;
        gaaval->bare_flag = 0; gaaval->din = 0; gaaval->ein=1; gaaval->hexflag = 0; gaaval->unlink_flag = 0;
-       gaaval->bit = 4; gaaval->keysize=0; gaaval->gzipflag=0; gaaval->bzipflag=0;
-       gaaval->sunlink_flag = 0; gaaval->keyfile=NULL; gaaval->algorithm=NULL; gaaval->keys=NULL;
-       gaaval->config_file=NULL; gaaval->mode=NULL; gaaval->input=NULL; gaaval->ed_specified=0 ;};
+       gaaval->kmode = NULL; gaaval->keysize=0; gaaval->gzipflag=0; gaaval->bzipflag=0;
+       gaaval->keyfile=NULL; gaaval->algorithm=NULL; gaaval->keys=NULL;
+       gaaval->config_file=NULL; gaaval->mode=NULL; gaaval->input=NULL; gaaval->ed_specified=0;
+       gaaval->double_check=0; gaaval->nolock=0; gaaval->noecho=1; gaaval->flush=0; ;};
 
     }
     inited = 1;
@@ -919,7 +911,6 @@ if(gaa_processing_file == 0)
 	GAA_INCOMP("zp");
 	GAA_INCOMP("Vq");
 	GAA_INCOMP("kf");
-	GAA_INCOMP("us");
 
 #line 507 "gaa.skel"
 #ifdef GAA_REST_EXISTS
@@ -936,8 +927,8 @@ if(gaa_processing_file == 0)
     default:
         printf("Unknown error\n");
     }
-}
 #endif
+}
     for(i = 1; i < argc; i++)
     {
         if(gaa_arg_used[i] == 0)
@@ -1017,7 +1008,7 @@ int gaa_internal_get_next_str(FILE *file, gaa_str_node *tmp_str, int argc)
     tmp_str->str[i - 1] = 0;
 
     fseek(file,- 1, SEEK_CUR);
-        
+    
     return -1;
 }
 
