@@ -35,6 +35,12 @@
 
 #include <defines.h>
 #include <random.h>
+#include "../libmcrypt/lib/xmemory.h"
+#include <extra.h>
+
+extern int noecho;
+
+static char rcsid[] = "$Id: getpass.c,v 1.1.1.1 1999/10/15 22:49:23 nmav Exp $";
 
 /* new code, #undef if there are any problems...  */
 #define USE_SETJMP 1
@@ -42,17 +48,17 @@
 #ifdef USE_SETJMP
 #include <setjmp.h>
 
-static sigjmp_buf intr;  /* where to jump on SIGINT */
+static sigjmp_buf intr;		/* where to jump on SIGINT */
 #endif
 
-static	int	sig_caught;
+static int sig_caught;
 #ifdef HAVE_SIGACTION
-static	struct	sigaction sigact;
+static struct sigaction sigact;
 #endif
 
-/*ARGSUSED*/
+/*ARGSUSED */
 static RETSIGTYPE
-sig_catch(int sig)
+ sig_catch(int sig)
 {
 	sig_caught = 1;
 #ifdef USE_SETJMP
@@ -60,13 +66,13 @@ sig_catch(int sig)
 #endif
 }
 
-#define MAXLEN 300
+#define MAXLEN MAX_KEY_LEN
 
-#ifndef OLD_READPASS
-static char *
-readpass(FILE *fp)
+char *
+ readpass(FILE * fp)
 {
-	static char input[MAXLEN + 1], asterix[MAXLEN + 1];
+	static char asterix[MAXLEN + 1];
+        char *input=mxmalloc(MAXLEN + 1);
 	char *cp, *ap, c;
 	int i;
 
@@ -77,9 +83,9 @@ readpass(FILE *fp)
 	ap = asterix;
 
 #ifdef HAVE_DEV_RANDOM
-        open_rand(0); /* We do not need much randomness here */
+	open_rand(0);		/* We do not need much randomness here */
 #endif
-        
+
 	while (read(fileno(fp), &c, 1)) {
 		switch (c) {
 		case '\n':
@@ -88,23 +94,24 @@ readpass(FILE *fp)
 		case '\b':
 		case 127:
 			if (cp > input) {
-				cp--; ap--;
+				cp--;
+				ap--;
 				for (i = 0; i < (*ap); i++) {
 					putc('\b', stderr);
 					putc(' ', stderr);
 					putc('\b', stderr);
 				}
 			} else
-				putc('\a', stderr);  /* BEL */
+				putc('\a', stderr);	/* BEL */
 			break;
 		default:
 			*cp++ = c;
 #ifdef HAVE_DEV_RANDOM
-			*ap++ = (get_o_rand8() % 4)+1;
+			*ap++ = (get_o_rand8() % 4) + 1;
 #else
-			*ap++ = (RAND8 % 4)+1;
-#endif			
-			for (i = 0; i < (*(ap-1)); i++)
+			*ap++ = (RAND8 % 4) + 1;
+#endif
+			for (i = 0; i < (*(ap - 1)); i++)
 				putc('*', stderr);
 			break;
 		}
@@ -113,39 +120,40 @@ readpass(FILE *fp)
 			break;
 	}
 
+      endwhile:
 #ifdef HAVE_DEV_RANDOM
-	close_rand(); /* close /dev/urandom */
-#endif	
-
-endwhile:
+	close_rand();		/* close /dev/urandom */
+#endif
 	*cp = 0;
 	putc('\n', stderr);
-        fflush(stderr);
+	fflush(stderr);
 	return input;
+
 }
-#else
-static char *
-readpass(FILE *fp)
+
+char *
+ readpass_old(FILE * fp)
 {
-	static char input[MAXLEN + 1];
+	char* input=mxmalloc(MAXLEN + 1);
+	char *cp;
 
 	fflush(stderr);
-	if (fgets(input, sizeof input, fp) == input) {
-		if ((cp = strchr (input, '\n')))
+	if (fgets(input, MAXLEN+1, fp) == input) {
+		if ((cp = strchr(input, '\n')))
 			*cp = '\0';
 		else
-			input[sizeof input - 1] = '\0';
+			input[MAXLEN] = '\0';
 #ifdef USE_SGTTY
 		putc('\n', stderr);
 #endif
 		return input;
+
 	}
 	return NULL;
 }
-#endif
 
 char *
-getpass(const char *prompt)
+ getpass(const char *prompt)
 {
 	static char nostring[1] = "";
 	static char *return_value;
@@ -154,7 +162,7 @@ getpass(const char *prompt)
 #ifdef HAVE_SIGACTION
 	struct sigaction old_sigact;
 #else
-	RETSIGTYPE (*old_signal)();
+	RETSIGTYPE(*old_signal) ();
 #endif
 	TERMIO new_modes;
 	TERMIO old_modes;
@@ -173,9 +181,9 @@ getpass(const char *prompt)
 	 * from stdin instead.
 	 */
 
-	if ((fp = fopen ("/dev/tty", "r")) == 0) {
+	if ((fp = fopen("/dev/tty", "r")) == 0) {
 		fp = stdin;
-		setbuf (fp, (char *) 0);
+		setbuf(fp, (char *) 0);
 	} else {
 		tty_opened = 1;
 	}
@@ -190,7 +198,6 @@ getpass(const char *prompt)
 		return_value = NULL;
 		goto out2;
 	}
-
 #ifdef USE_SETJMP
 	/*
 	 * If we get a SIGINT, sig_catch() will jump here -
@@ -206,19 +213,19 @@ getpass(const char *prompt)
 	sigact.sa_flags = 0;
 	sigaction(SIGINT, &sigact, &old_sigact);
 #else
-	old_signal = signal (SIGINT, sig_catch);
+	old_signal = signal(SIGINT, sig_catch);
 #endif
 
 	new_modes = old_modes;
 
 #ifdef USE_SGTTY
-	new_modes.sg_flags &= ~ECHO ;
+	new_modes.sg_flags &= ~ECHO;
 #else
-#ifdef OLD_READPASS
-	new_modes.c_lflag &= ~(ECHO|ECHOE|ECHOK);
-#else
-	new_modes.c_lflag &= ~(ECHO|ECHOE|ECHOK|ICANON);
-#endif
+  if (noecho==TRUE) {
+	new_modes.c_lflag &= ~(ECHO | ECHOE | ECHOK);
+  } else {
+	new_modes.c_lflag &= ~(ECHO | ECHOE | ECHOK | ICANON);
+  }
 	new_modes.c_lflag |= ECHONL;
 #endif
 
@@ -235,8 +242,12 @@ getpass(const char *prompt)
 	if (fputs(prompt, stderr) == EOF)
 		goto out;
 
+      if (noecho==FALSE) {
 	return_value = readpass(fp);
-out:
+      } else {
+      	return_value = readpass_old(fp);
+      }
+      out:
 	/*
 	 * the old SIGINT handler is restored after the tty
 	 * modes.  then /dev/tty is closed if it was opened in
@@ -248,13 +259,13 @@ out:
 		return_value = NULL;
 
 #ifdef HAVE_SIGACTION
-	(void) sigaction (SIGINT, &old_sigact, NULL);
+	(void) sigaction(SIGINT, &old_sigact, NULL);
 #else
-	(void) signal (SIGINT, old_signal);
+	(void) signal(SIGINT, old_signal);
 #endif
-out2:
+      out2:
 	if (tty_opened)
-		(void) fclose (fp);
+		(void) fclose(fp);
 
 	if (sig_caught) {
 		kill(getpid(), SIGINT);
