@@ -1,11 +1,6 @@
-#ifndef DEFINES_H
- #define DEFINES_H
- #include <defines.h>
-#endif
-
 /* mcrypt.c */
 int encrypt_ucrypt(char *fromfile,char *tofile, char *key);
-void usage(char *);
+
 int encrypt_general(int algorithm, char *fromfile,char *tofile,char *key);
 int decrypt_general(int algorithm, char *fromfile,char *tofile,char *key);
 
@@ -17,3 +12,9 @@ int check_mode(char *chain);
 /* Solaris des(3) */
 int encrypt_desc(char *fromfile, char *tofile, char *key);
 int decrypt_desc(char *fromfile, char *tofile, char *key);
+
+void rol_buf(void * buffer, int buffersize,void * value);
+
+void mcrypt_version();
+void mcrypt_license();
+void usage(void);
